﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace ColourVoting
{
    public static class Extensions
    {
        public static IEnumerable<CountedGroup<TKey>> GroupAndCount<TSource, TKey>
            (this IEnumerable<TSource> source, Func<TSource, TKey> keySelector)
        {
            var groups = new Dictionary<TKey, CountedGroup<TKey>>();
            foreach (TSource element in source)
            {
                TKey key = keySelector(element);
                CountedGroup<TKey> group;
                if (!groups.TryGetValue(key, out group))
                {
                    group = new CountedGroup<TKey>(key);
                    groups[key] = group;
                }
                group.Increment();
            }
            foreach (CountedGroup<TKey> group in groups.Values)
            {
                yield return group;
            }
        }
    }

    public class CountedGroup<T>
    {
        private readonly T key;
        private int count;

        public T Key { get { return key; } }
        public int Count { get { return count; } }

        public CountedGroup(T key)
        {
            this.key = key;
        }

        internal void Increment()
        {
            count++;
        }
    }

    [Description("Limit the aggregation to counting, but stick to normal model")]
    class HardCodedGroupAndCount
    {
        static void Main()
        {
            var query = Voter.AllVoters()
                             .GroupAndCount(voter => voter.FavouriteColour.ToUpper());

            foreach (var entry in query)
            {
                Console.WriteLine("Colour {0} has {1} votes", entry.Key, entry.Count);
            }
        }
    }
}
