﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using Google.ProtocolBuffers;
using Logging.Properties;
using System.IO;
using MiscUtil.Linq;
using MiscUtil.Linq.Extensions;

namespace Logging
{
    public class PushLinq
    {
        static void Main()
        {
            // Look ma, loads of aggregates and only one pass through the data, with no buffering!
            Stopwatch sw = Stopwatch.StartNew();
            DataProducer<LogEntry> producer = new DataProducer<LogEntry>();

            AddProgressBar(producer, 10000);

            IFuture<int> count = producer.Count();

            var severityGroups = (from entry in producer
                                 group 1 by entry.Severity into groups
                                 select new { Severity = groups.Key, Count = groups.Count() })
                                .AsFutureEnumerable();

            var userGroups = (from entry in producer
                             group entry by entry.User into groups
                             select new { User = groups.Key, Count = groups.Count() })
                            .AsFutureEnumerable();

            string dir = Settings.Default.LogDirectory;
            var source = from file in Directory.GetFiles(dir, "*.log")
                         from entry in MessageStreamIterator<LogEntry>.FromFile(Path.Combine(dir, file))
                         select entry;

            source = source.Take(5000000);

            producer.ProduceAndEnd(source);
            sw.Stop();
            Console.WriteLine("Elapsed ms: {0}", sw.ElapsedMilliseconds);

            Console.WriteLine("Total count: {0}", count.Value);
            Console.WriteLine("Count by severity: ");
            foreach (var result in severityGroups.Value)
            {
                Console.WriteLine("  {0}: {1}", result.Severity, result.Count.Value);
            }
            Console.WriteLine("Count by user: ");
            foreach (var result in userGroups.Value)
            {
                Console.WriteLine("  {0}: {1}", result.User, result.Count.Value);
            }
        }

        private static void AddProgressBar<T>(IDataProducer<T> producer, int period)
        {
            int count = 0;
            producer.DataProduced += data =>
            {
                count++;
                if (count == period)
                {
                    Console.Write(".");
                    Console.Out.Flush();
                    count = 0;
                }
            };
            producer.EndOfData += () => Console.WriteLine();
        }
    }
}
